
import PyML
version = tuple([int(token) for token in PyML.__version__.split('.')])
if version < (0,7) :
    raise SystemExit, "version 0.7 or higher of PyML is required."

from PyML.containers import VectorDataSet,SparseDataSet,SequenceData
from PyML.containers.sequenceData import spectrumData
from PyML.containers import Labels
from PyML.classifiers import SVM

def main() :

    acceptor_results = splice_example('C_elegans_acc_gc.csv',
                                      'C_elegans_acc.fasta')

    print '\n******** results can be found in the file results.txt ********'
    keys = acceptor_results.keys()
    keys.sort()
    f = open('results.txt','w')
    f.write('Kernel\tAcceptor Results\n');
    for key in keys :
        f.write('%s\t' % key)
        f.write('%2.1f%%\n' % (100.0*acceptor_results[key].roc))
    f.close()

def process_header(header) :
    """
    extract sequence ID and its label from the fasta file
    """
    id,labelToken = header.split()
    return id.strip(),labelToken.split('=')[1]

def gen_spectrum_data(filename, k1, k2=None, normalize=False) :

    seqdata = SequenceData(filename, headerHandler = process_header)
    seq1 = [seqdata.getSequence(i)[:40] for i in range(len(seqdata))]
    seq2 = [seqdata.getSequence(i)[42:] for i in range(len(seqdata))]

    data1 = spectrumData(seq1, k1, k2, addon = '1', normalize=normalize)
    data2 = spectrumData(seq2, k1, k2, addon = '2', normalize=normalize)
    data1.addFeatures(data2)
    data1.attachLabels(seqdata.labels)
    if normalize :
        data1.normalize()

    return data1


def run_gc(gcfilename, num_folds=5, C=1, seed=1) :

    print gcfilename
    data = VectorDataSet(gcfilename, labelsColumn = 0)

    from PyML.preproc import preproc
    standardizer = preproc.Standardizer()
    standardizer.train(data)

    results = {}

    # Linear kernel
    results['gc linear kernel'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    # Polynomial kernel
    data.attachKernel('poly', degree = 2)
    results['gc poly degree 2'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    data.attachKernel('poly', degree = 5)
    results['gc poly degree 5'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    # Gaussian kernel
    data.attachKernel('gaussian', gamma = 1/0.1)
    results['gc gauss width 0.1'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    data.attachKernel('gaussian', gamma = 1/1.0)
    results['gc gauss width 1.0'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    data.attachKernel('gaussian', gamma = 1/10.0)
    results['gc gauss width 10.0'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    return results


def run_spectrum(seqfilename, num_folds=5, C=1, seed=1) :

    results = {}

    data = gen_spectrum_data(seqfilename, 1, 1, True)
    results['spectrum 1'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    data = gen_spectrum_data(seqfilename, 3, 3, True)
    results['spectrum 3'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    data = gen_spectrum_data(seqfilename, 5, 5, True)
    results['spectrum 5'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    #mixed Spectrum kernel

    data = gen_spectrum_data(seqfilename, 1, 3)
    results['spectrum (mixed) 1-3'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    data = gen_spectrum_data(seqfilename, 1, 5)
    results['spectrum (mixed) 1-5'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    return results

def run_wd(seqfilename, num_folds=5, C=1, seed=1) :

    results = {}

    data = SequenceData(seqfilename, mink = 1, maxk = 1, maxShift=0, 
                        headerHandler = process_header)
    data.attachKernel('cosine')
    results['wd 1'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    data = SequenceData(seqfilename, mink = 1, maxk = 3, maxShift=0, 
                        headerHandler = process_header)
    data.attachKernel('cosine')
    results['wd 3'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)

    data = SequenceData(seqfilename, mink = 1, maxk = 5, maxShift=0, 
                        headerHandler = process_header)
    data.attachKernel('cosine')
    results['wd 5'] = SVM(C=C).stratifiedCV(data, numFolds=num_folds, seed=seed)


    return results

def splice_example(gcfilename, seqfilename) :
    """
    generate the results for each of the kernels: 
    gc - features based on gc content
    spectrum - the spectrum kernel
    wd - the weighted degree kernel
    """

    num_folds = 5
    C = 1
    results = {}
    results.update(run_gc(gcfilename, num_folds, C))
    results.update(run_spectrum(seqfilename, num_folds, C))
    results.update(run_wd(seqfilename, num_folds, C))
    
    return results
                   
if __name__ == '__main__' :

    main()
